--- loveui is a love2d library to provide resuable GUI widgets to love2d 
-- developers.

module ("ui", package.seeall)

polygon = love.graphics.polygon
rectangle = love.graphics.rectangle
push = love.graphics.push
pop = love.graphics.pop
rotate = love.graphics.rotate
translate = love.graphics.translate

--- Draw a sub-part of a circle
-- @param x The center x coordinate.
-- @param y The center y coordinate.
-- @param angle The angle to rotate the arc.
-- @param radius The radius from the outer side of the arc.
-- @param width The thickness of the arc.
-- @param length The length of the arc in radians.
function arc(x, y, angle, radius, width, length)
  -- Width cannot be larger than radius.
  width = math.min(radius, width)
  
  -- Angle must be between 0 and 2 PI
  angle = math.max(math.min(math.pi * 2, angle), 0)
  
  -- Precision of the round edge.
  local delta = 0.025*math.pi
  
  -- Trigonometry
  local function pointx(radius, angle, centerx)
    return radius*math.cos(angle)+centerx
  end
  local function pointy(radius, angle, centery)
    return radius*math.sin(angle)+centery
  end
  
  local last = angle
  
  -- Draw arc by drawing many little trapezoids.
  for current = angle+delta, angle+length, delta do
    polygon("fill",
      pointx(radius-width, last, x), pointy(radius-width, last, y),
      pointx(radius, last, x), pointy(radius, last, y),
      pointx(radius, current, x), pointy(radius, current, y),
      pointx(radius-width, current, x),pointy(radius-width, current, y))
    last = current
  end
end